@@ -75,7 +75,7 @@ void read_history_old ()
   if (history_file)   {
     /* Read the size of the first item */
     gint size=1;
-    g_mutex_lock(hist_lock);
+    g_mutex_lock(&hist_lock);
     /* Continue reading until size is 0 */
     while (size)   {
 			struct history_item *c;
@@ -102,7 +102,7 @@ void read_history_old ()
     /* Close file and reverse the history to normal */
     fclose(history_file);
     history_list = g_list_reverse(history_list);
-		g_mutex_unlock(hist_lock);
+		g_mutex_unlock(&hist_lock);
   }
 }
 
@@ -170,6 +170,7 @@ Current scheme is to have the total zize of element followed by the type, then t
 void read_history ()
 {
 	size_t x;
+	int is_locked=0;
   /* Build file path */
   gchar* history_path = g_build_filename(g_get_user_data_dir(),HISTORY_FILE0,NULL); 
 	gchar *magic=g_malloc0(2+HISTORY_MAGIC_SIZE);
@@ -195,7 +196,8 @@ void read_history ()
 		}
 		if(dbg) g_printf("History Magic OK. Reading\n");
     /* Continue reading until size is 0 */
-		g_mutex_lock(hist_lock);
+		g_mutex_lock(&hist_lock);
+		is_locked=1;
     while (size)   {
 			struct history_item *c;
 			if (fread(&size, 4, 1, history_file) != 1)
@@ -229,7 +231,8 @@ done:
     /* Close file and reverse the history to normal */
     fclose(history_file);
     history_list = g_list_reverse(history_list);
-		g_mutex_unlock(hist_lock);
+		if(is_locked)
+			g_mutex_unlock(&hist_lock);
   }
 	if(dbg) g_printf("History read done\n");
 }
@@ -278,7 +281,7 @@ void save_history()
 		}	
 		memcpy(magic,history_magics[HISTORY_VERSION-1],strlen(history_magics[HISTORY_VERSION-1]));	
 		fwrite(magic,HISTORY_MAGIC_SIZE,1,history_file);
-		g_mutex_lock(hist_lock);
+		g_mutex_lock(&hist_lock);
     /* Write each element to a binary file */
     for (element = history_list; element != NULL; element = element->next) {
 		  struct history_item *c;
@@ -297,7 +300,7 @@ void save_history()
 			}
 			
     }
-		g_mutex_unlock(hist_lock);
+		g_mutex_unlock(&hist_lock);
     /* Write 0 to indicate end of file */
     gint end = 0;
     fwrite(&end, 4, 1, history_file);
@@ -370,7 +373,7 @@ void append_item(gchar* item, int checkdup, gint iflags, gint itype)
 	struct history_item *c=NULL;
 	if(NULL == item)
 		return;
-	g_mutex_lock(hist_lock);
+	g_mutex_lock(&hist_lock);
 /**delete if HIST_DEL flag is set.  */
 	if( checkdup & HIST_CHECKDUP){
 		node=is_duplicate(item, checkdup & HIST_DEL, &flags);
@@ -400,7 +403,7 @@ void append_item(gchar* item, int checkdup, gint iflags, gint itype)
 	/*g_printf("Append '%s'\n",item); */
    /* Prepend new item */
   history_list = g_list_prepend(history_list, NULL ==element?c:element->data);
-	g_mutex_unlock(hist_lock);
+	g_mutex_unlock(&hist_lock);
   /* Shorten history if necessary */
   truncate_history();
 }
@@ -415,7 +418,7 @@ void delete_duplicate(gchar* item)
   GList* element;
 	int p=get_pref_int32("persistent_history");
   /* Go through each element compare each */
-	g_mutex_lock(hist_lock);
+	g_mutex_lock(&hist_lock);
   for (element = history_list; element != NULL; element = element->next) {
 	  struct history_item *c;
 		c=(struct history_item *)element->data;
@@ -428,7 +431,7 @@ void delete_duplicate(gchar* item)
 	    }
 		}
   }
-	g_mutex_unlock(hist_lock);
+	g_mutex_unlock(&hist_lock);
 }
 
 /***************************************************************************/
@@ -442,7 +445,7 @@ void truncate_history()
 {
   int p=get_pref_int32("persistent_history");
   if (history_list)  {
-		g_mutex_lock(hist_lock);
+		g_mutex_lock(&hist_lock);
 		guint ll=g_list_length(history_list);
 		guint lim=get_pref_int32("history_limit");
 		if(ll > lim){ /* Shorten history if necessary */
@@ -456,7 +459,7 @@ void truncate_history()
 				}
 	    }	
 		}
-		g_mutex_unlock(hist_lock);
+		g_mutex_unlock(&hist_lock);
     /* Save changes */
     if (get_pref_int32("save_history"))
       save_history();
@@ -489,7 +492,7 @@ gpointer get_last_item()
 ****************************************************************************/
 void clear_history( void )
 {
-	g_mutex_lock(hist_lock);
+	g_mutex_lock(&hist_lock);
 	if( !get_pref_int32("persistent_history")){
 		g_list_free(history_list);
 		history_list = NULL;
@@ -503,34 +506,41 @@ void clear_history( void )
 				history_list=g_list_remove(history_list,c);
 		}		
 	}
-	g_mutex_unlock(hist_lock);
+	g_mutex_unlock(&hist_lock);
   if (get_pref_int32("save_history"))
   	save_history();
 }
 /***************************************************************************/
 /** .
 \n\b Arguments:
+\n\b path - path to file to save
+\n\b mode - 1 to print line numbers.
 \n\b Returns:
 ****************************************************************************/
-int save_history_as_text(gchar *path)
+int save_history_as_text(gchar *path, int mode)
 {
 	FILE* fp = fopen(path, "w");
   /* Check that it opened and begin write */
   if (fp)  {
 		gint i;
     GList* element;
-		g_mutex_lock(hist_lock);
+		gchar *fstring="%s\n";
+		g_mutex_lock(&hist_lock);
     /* Write each element to  file */
     for (i=0,element = history_list; element != NULL; element = element->next) {
 		  struct history_item *c;
 			c=(struct history_item *)element->data;
-			if(c->flags & CLIP_TYPE_PERSISTENT)
-				fprintf(fp,"PHIST_%04d %s\n",i,c->text);
-			else
-				fprintf(fp,"NHIST_%04d %s\n",i,c->text);
+			if(mode){
+				if(c->flags & CLIP_TYPE_PERSISTENT)
+					fstring="PHIST_%04d %s\n";
+				else
+					fstring="NHIST_%04d %s\n";	
+				fprintf(fp,fstring,i,c->text);
+			}else
+				fprintf(fp,fstring,c->text);
 			++i;
     }
-		g_mutex_unlock(hist_lock);
+		g_mutex_unlock(&hist_lock);
     fclose(fp);
   }
 	
@@ -562,7 +572,7 @@ void history_save_as(GtkMenuItem *menu_item, gpointer user_data)
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
 	    gchar *filename;
 	    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
-	    save_history_as_text (filename);
+	    save_history_as_text (filename, get_pref_int32("save_history_Lineno"));
 	    g_free (filename);
 	  }
 	gtk_widget_destroy (dialog);
