@@ -66,6 +66,8 @@ g_signal_connect(clipboard, "owner-change",  G_CALLBACK(handle_owner_change), NU
 
 #include <ctype.h>
 #include <pthread.h>
+#include <libgen.h>
+
 
 /**ACT are actions, and MODE is the mode of the action  */
 /** #define ACT_STOP  0
@@ -87,6 +89,7 @@ g_signal_connect(clipboard, "owner-change",  G_CALLBACK(handle_owner_change), NU
 #define FIFCMD_RUN_PRI "run_pri"
 #define FIFCMD_RUN_CLI "run_cli"
 #define FIFCMD_RUN_ALL "run_all"
+#define FIFCMD_CLEAR_ALL "clear_all"
 
 GtkWidget *hmenu;
 /**see parcellite.h for DEBUG defines  */
@@ -98,7 +101,7 @@ static AppIndicator *indicator=NULL;
 static GtkWidget *indicator_menu = NULL;
 #endif
 static GtkStatusIcon *status_icon=NULL; 
-GMutex *hist_lock=NULL;
+GMutex hist_lock;
 static gboolean actions_lock = FALSE;
 static int show_icon=0;
 static int have_appindicator=0; /**if set, we have a running indicator-appmenu  */
@@ -515,6 +518,7 @@ void update_clipboards(gchar *intext, gint mode)
 	update_clipboard(clipboard, intext, mode);
 }
 
+static void clear_all ( gpointer user_data);
 /***************************************************************************/
 /** Run a command. For now, just start and stop
 \n\b Arguments:
@@ -522,32 +526,55 @@ void update_clipboards(gchar *intext, gint mode)
 ****************************************************************************/
 void do_command(gchar *buf, gint len)
 {
-  g_printf("Got '%s' cmd\n",buf);
-	if(!p_strcmp(buf,FIFCMD_RUN_ALL)) {
+	/**clean whitepsace  */
+	gchar *p, *news=strdup(buf); 
+	p=g_strchomp(news);
+	int saveme=0;
+  g_fprintf(stderr,"Got '%s' cmd\n",p);
+	if(0 == strncmp(p,"save_historyN ", 14)) 
+		saveme=14;
+	else if(0 == strncmp(p,"save_history ", 13))
+		saveme=13;
+	if(saveme){
+		gchar *tmp=strdup(&p[saveme]);
+		gchar *dir=dirname(tmp);
+		if(0 == access(dir,W_OK)){
+			save_history_as_text(&p[saveme],13==saveme?0:1);
+		}
+		g_free(tmp);
+		goto end;
+	}
+		
+	if(!p_strcmp(p,FIFCMD_RUN_ALL)) {
 		cmd_mode|=CMODE_ALL;
-		return;
+		goto end;
 	}	
-	if(!p_strcmp(buf,FIFCMD_RUN_CLI)) {
+	if(!p_strcmp(p,FIFCMD_RUN_CLI)) {
 		cmd_mode|=CMODE_CLI;
-		return;
+		goto end;
 	}	
-	if(!p_strcmp(buf,FIFCMD_RUN_PRI)) {
+	if(!p_strcmp(p,FIFCMD_RUN_PRI)) {
 		cmd_mode|=CMODE_PRI;
-		return;
+		goto end;
 	}	
-	if(!p_strcmp(buf,FIFCMD_STOP_ALL)) {
+	if(!p_strcmp(p,FIFCMD_STOP_ALL)) {
 		cmd_mode&=~(CMODE_ALL);
-		return;
+		goto end;
 	}
-		if(!p_strcmp(buf,FIFCMD_STOP_CLI)) {
+	if(!p_strcmp(p,FIFCMD_STOP_CLI)) {
 		cmd_mode&=~(CMODE_CLI);
-		return;
+		goto end;
 	}
-		if(!p_strcmp(buf,FIFCMD_STOP_PRI)) {
+	if(!p_strcmp(p,FIFCMD_STOP_PRI)) {
 		cmd_mode&=~(CMODE_PRI);
-		return;
+		goto end;
 	}
-	
+	if(!p_strcmp(p,FIFCMD_CLEAR_ALL)) {
+		clear_all((gpointer) NULL);
+		goto end;
+	}
+end:
+	free(news);
 }
 
 /***************************************************************************/
@@ -563,9 +590,9 @@ void check_clipboards(gint mode)
 	/*g_printf("check_clipboards\n"); */
 	if(fifo->clen){/**we have a command to execute  */
 			/*fifo->which should be ID_CMD: */
-		if(fifo->dbg) g_printf("Running CMD '%s'\n",fifo->cbuf);
+		if(fifo->dbg) g_fprintf(stderr,"Running CMD '%s'\n",fifo->cbuf);
 		do_command(fifo->cbuf, fifo->clen);
-		if(fifo->dbg) g_printf("mode is 0x%X\n",cmd_mode);
+		if(fifo->dbg) g_fprintf(stderr,"mode is 0x%X\n",cmd_mode);
 		fifo->clen=0;
 		return;
 	}	
@@ -575,14 +602,14 @@ void check_clipboards(gint mode)
 		switch(fifo->which){
 			case ID_PRIMARY:
 				fifo->rlen=validate_utf8_text(fifo->buf, fifo->rlen);
-				if(fifo->dbg) g_printf("Setting PRI '%s'\n",fifo->buf);
+				if(fifo->dbg) g_fprintf(stderr,"Setting PRI '%s'\n",fifo->buf);
 				update_clipboard(primary, fifo->buf, H_MODE_NEW);
 				fifo->rlen=0;
 				n=1;
 				break;
 			case ID_CLIPBOARD:
 				fifo->rlen=validate_utf8_text(fifo->buf, fifo->rlen);
-				if(fifo->dbg) g_printf("Setting CLI '%s'\n",fifo->buf);
+				if(fifo->dbg) g_fprintf(stderr,"Setting CLI '%s'\n",fifo->buf);
 				update_clipboard(clipboard, fifo->buf, H_MODE_NEW);
 				n=2;
 				fifo->rlen=0;
@@ -590,7 +617,7 @@ void check_clipboards(gint mode)
 			
 			default:
 				fifo->rlen=validate_utf8_text(fifo->buf, fifo->rlen);
-				g_printf("CLIP not set, discarding '%s'\n",fifo->buf);
+				/*g_fprintf(stderr,"CLIP not set, discarding '%s'\n",fifo->buf); */
 				fifo->rlen=0;
 				break;
 		}
@@ -631,7 +658,8 @@ gboolean check_for_appindictor( gpointer data)
 		/*g_printf("Looking for '%s'\n",appindicator_process); */
 		if(get_pref_int32("multi_user"))
 			mode|=PROC_MODE_USER_QUALIFY;
-		if(proc_find(appindicator_process,mode,NULL) >0){
+		
+		if(get_pref_int32("on_unity") || proc_find(appindicator_process,mode,NULL) >0 ){
 			have_appindicator=1;
 			if(NULL == indicator && show_icon)
 				create_app_indicator();	
@@ -664,29 +692,6 @@ gboolean check_clipboards_tic(gpointer data)
 #endif
 	return TRUE;
 }
-#if 0
-/* Thread function called for each action performed */
-static void *execute_action(void *command)
-{
-  /* Execute action */
-  actions_lock = TRUE;
-  if (!have_appindicator && show_icon) {
-  gtk_status_icon_set_from_stock((GtkStatusIcon*)status_icon, GTK_STOCK_EXECUTE);
-  gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Executing action..."));
-  }
-  if(system((gchar*)command))
-  	g_fprintf(stderr,"sytem command '%s' failed\n",(gchar *)command);
-  if (!have_appindicator &&show_icon) {
-	gtk_status_icon_set_from_icon_name((GtkStatusIcon*)status_icon, get_pref_string("icon_name"));
-  gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Clipboard Manager"));
-  }
-  actions_lock = FALSE;
-  g_free((gchar*)command);
-  /* Exit this thread */
-  pthread_exit(NULL);
-	return NULL;
-}
-#endif
 /* Called when execution action exits */
 static void action_exit(GPid pid, gint status, gpointer data)
 {
@@ -709,23 +714,24 @@ static void action_selected(GtkButton *button, gpointer user_data)
   }
   /* Insert clipboard into command (user_data), and prepare it for execution */
   gchar* clipboard_text = gtk_clipboard_wait_for_text(clipboard);
-	g_fprintf(stderr,"Got cmd '%s', text '%s'->",(gchar *)user_data,clipboard_text);fflush(NULL);  
+	/*g_fprintf(stderr,"Got cmd '%s', text '%s'->",(gchar *)user_data,clipboard_text);fflush(NULL);   */
 	gchar* quoted_clipboard_text = g_shell_quote(clipboard_text);
 	gchar* command=g_strdup_printf((gchar *)user_data,quoted_clipboard_text);
-	g_fprintf(stderr," '%s'\n",command);fflush(NULL);  
+	//g_fprintf(stderr," '%s'->",command);
   g_free(clipboard_text);
   g_free(quoted_clipboard_text);
   g_free(user_data);
   gchar* shell_command = g_shell_quote(command);
   g_free(command);
+	
   gchar* cmd = g_strconcat("/bin/sh -c ", shell_command, NULL);
   g_free(shell_command);
-  
+  //g_fprintf(stderr," '%s'\n",cmd);fflush(NULL);  
   /* Execute action */
   GPid pid;
   gchar **argv;
   g_shell_parse_argv(cmd, NULL, &argv, NULL);
-	g_fprintf(stderr,"cmd '%s' argv '%s' '%s' '%s'\n",cmd,argv[1],argv[2],argv[3]);  
+	/*g_fprintf(stderr,"cmd '%s' argv '%s' '%s' '%s'\n",cmd,argv[1],argv[2],argv[3]);   */
   g_free(cmd);
   g_spawn_async(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL);
   g_child_watch_add(pid, (GChildWatchFunc)action_exit, NULL);
@@ -774,7 +780,7 @@ static void edit_selected(GtkMenuItem *menu_item, gpointer user_data)
 				return;
 			}
 			current_clipboard_text=p_strdup(c->text);
-			g_fprintf(stderr,"Got Text frmo wi.index ele %p '%s'",element,c->text);  
+			/*g_fprintf(stderr,"Got Text frmo wi.index ele %p '%s'",element,c->text);   */
 			
 		}	else{
 			h->wi.tmp1=0;
@@ -789,7 +795,7 @@ static void edit_selected(GtkMenuItem *menu_item, gpointer user_data)
 		}
     
     if (current_clipboard_text != NULL)   {
-		g_fprintf(stderr,"Got '%s' for edit text\n",current_clipboard_text); 
+		/*g_fprintf(stderr,"Got '%s' for edit text\n",current_clipboard_text);  */
 			TRACE(g_fprintf(stderr,"Got '%s'\n",current_clipboard_text));
       gtk_text_buffer_set_text(text_buffer, current_clipboard_text, -1);
     }	 else	 {
@@ -1066,10 +1072,30 @@ void  history_item_right_click (struct history_info *h, GdkEventKey *e, gint ind
 	
 }
 
+/***************************************************************************/
+/** Clear history, without confirmation.
+\n\b Arguments:
+\n\b Returns:
+****************************************************************************/
+static void clear_all ( gpointer user_data)
+{
+	if( NULL != user_data ){ 
+		struct history_info *h=(struct history_info *)user_data;
+		/* Clear history and free history-related variables */
+		remove_deleted_items(h); /**fix bug 92, Shift/ctrl right-click followed by clear segfaults/double free.  */	
+	}	
+	clear_history();
+	/*g_printf("Clear hist done, h=%p, h->delete_list=%p\n",h, h->delete_list); */
+	update_clipboard(primary, "", H_MODE_INIT);
+  update_clipboard(clipboard, "", H_MODE_INIT);	
+}
+
 /* Called when Clear is selected from history menu */
 static void clear_selected(GtkMenuItem *menu_item, gpointer user_data)
 {
 	int clear=1;
+	if(get_pref_int32("disable_clear"))
+		return;
   /* Check for confirm clear option */
   if (get_pref_int32("confirm_clear"))  {
     GtkWidget* confirm_dialog = gtk_message_dialog_new(NULL,
@@ -1084,13 +1110,7 @@ static void clear_selected(GtkMenuItem *menu_item, gpointer user_data)
 		gtk_widget_destroy(confirm_dialog);
 	}	
 	if(clear){
-		struct history_info *h=(struct history_info *)user_data;
-		/* Clear history and free history-related variables */
-		remove_deleted_items(h); /**fix bug 92, Shift/ctrl right-click followed by clear segfaults/double free.  */	
-		clear_history();
-		/*g_printf("Clear hist done, h=%p, h->delete_list=%p\n",h, h->delete_list); */
-		update_clipboard(primary, "", H_MODE_INIT);
-    update_clipboard(clipboard, "", H_MODE_INIT);
+		clear_all(user_data);
 	}
       
 }
@@ -1230,7 +1250,7 @@ static gboolean show_actions_menu(gpointer data)
   gtk_menu_shell_append((GtkMenuShell*)menu, gtk_separator_menu_item_new());
   /* Actions */
   gchar* path = g_build_filename(g_get_user_data_dir(), ACTIONS_FILE, NULL);
-	printf("got path '%s'\n",path); fflush(NULL);
+	g_fprintf(stderr,"got path '%s'\n",path); fflush(NULL);
   FILE* actions_file = fopen(path, "rb");
   g_free(path);
   /* Check that it opened and begin read */
@@ -1238,7 +1258,7 @@ static gboolean show_actions_menu(gpointer data)
   {
     gint size;
     if(0==fread(&size, 4, 1, actions_file))
-    	g_print("1:got 0 items from fread\n");
+    	g_fprintf(stderr,"1:got 0 items from fread\n");
     /* Check if actions file is empty */
     if (!size)
     {
@@ -1253,20 +1273,20 @@ static gboolean show_actions_menu(gpointer data)
       /* Read name */
       gchar* name = (gchar*)g_malloc(size + 1);
       if( 0 ==fread(name, size, 1, actions_file))
-      	g_print("2:got 0 items from fread\n");
+      	g_fprintf(stderr,"2:got 0 items from fread\n");
       name[size] = '\0';
       menu_item = gtk_menu_item_new_with_label(name);
       
       if(0 ==fread(&size, 4, 1, actions_file))
-      	g_print("3:got 0 items from fread\n");
+      	g_fprintf(stderr,"3:got 0 items from fread\n");
       /* Read command */
       gchar* command = (gchar*)g_malloc(size + 1);
       if(0 ==fread(command, size, 1, actions_file))
-      	g_print("4:got 0 items from fread\n");
+      	g_fprintf(stderr,"4:got 0 items from fread\n");
       command[size] = '\0';
 		  g_print("name='%s' cmd='%s'\n",name,command);
       if(0 ==fread(&size, 4, 1, actions_file))
-      	g_print("5:got 0 items from fread\n");
+      	g_fprintf(stderr,"5:got 0 items from fread\n");
       /* Append the action */
       gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
       g_signal_connect((GObject*)menu_item,        "activate",
@@ -1420,15 +1440,15 @@ static gboolean key_release_cb (GtkWidget *w,GdkEventKey *e, gpointer user)
 	if(0&& NULL != e ){
 		if(GDK_MOTION_NOTIFY==e->type)
 			return FALSE;
-    printf("krc (%x) S%x T%x C%x,SE%x, G%x, W%p, wdg%p",
+    g_fprintf(stderr,"krc (%x) S%x T%x C%x,SE%x, G%x, W%p, wdg%p",
 		e->keyval,e->state,e->type,
 		e->hardware_keycode,e->send_event,e->group,e->window,w);
 		if(GDK_DRAG_ENTER == e->type || GDK_DRAG_LEAVE==e->type){
-			printf(" Drag %s\n",GDK_DRAG_ENTER == e->type?"ENTER":"LEAVE");
+			g_fprintf(stderr," Drag %s\n",GDK_DRAG_ENTER == e->type?"ENTER":"LEAVE");
 		}
 		if(GDK_BUTTON_RELEASE==e->type || GDK_BUTTON_PRESS==e->type  ){
 			GdkEventButton *b=(GdkEventButton *)e;
-			printf(" button %d State 0x%x\n",b->button, b->state);
+			g_fprintf(stderr," button %d State 0x%x\n",b->button, b->state);
 			if(GDK_BUTTON_RELEASE==e->type && 3 == b->button){
 				/*toggle-size-request", */
 				/*allow item_selected to get called  */
@@ -1436,7 +1456,7 @@ static gboolean key_release_cb (GtkWidget *w,GdkEventKey *e, gpointer user)
 			}
 			/*return TRUE; */
 		}
-		printf("\n");
+		g_fprintf(stderr,"\n");
 		fflush(NULL);
   }
 	
@@ -1448,11 +1468,11 @@ static gboolean key_release_cb (GtkWidget *w,GdkEventKey *e, gpointer user)
 		idx=0;
 		return FALSE;
 	}else if(NULL == kstr){
-		g_print("kstr null. Not init\n");
+		g_fprintf(stderr,"kstr null. Not init\n");
 		return FALSE;
 	}
   if(NULL == e){
-    g_print("No Event!\n");
+    g_fprintf(stderr,"No Event!\n");
     return FALSE;
   }
 		
@@ -1592,6 +1612,26 @@ foundit:
 	TRACE(g_fprintf(stderr,"\n"));
 	return TRUE;
 }	
+extern char **environ;
+/***************************************************************************/
+/** Get environment vars
+\n\b Arguments:
+\n\b Returns:
+****************************************************************************/
+static gchar **get_environment( char *txt )
+{
+	gint i,l;
+	char **env;
+	for (l=0; NULL != environ[l]; ++l);
+	l+=2;/* allow for NULL and our env */
+	if(NULL == (env=malloc(l*sizeof(gchar *))))
+		return NULL;
+	for (i=0; i<l-2; ++i)
+		env[i]=environ[i];
+	env[i++]=txt;
+	env[i]=NULL;
+	return env;
+}
 
 /***************************************************************************/
 /** Set clipboard from history list.
@@ -1604,6 +1644,8 @@ void set_clipboard_text(struct history_info *h, GList *element)
 	gchar *action=NULL;
 	gchar *txt=NULL;
 	gchar *cmd=NULL;
+	gchar **env=NULL;
+	
 	if(NULL == find_h_item(h->delete_list,NULL,element)){	/**not in our delete list  */
 		/**make a copy of txt, because it gets freed and re-allocated.  */
 		txt=p_strdup(((struct history_item *)(element->data))->text);
@@ -1611,7 +1653,7 @@ void set_clipboard_text(struct history_info *h, GList *element)
 		if(use_copy)
 			update_clipboard(clipboard, txt, H_MODE_LIST);
 		if(use_primary)
-	  	update_clipboard(primary, txt, H_MODE_LIST);	
+	  	    update_clipboard(primary, txt, H_MODE_LIST);	
 		
 		auto_whatever=1;
 	}
@@ -1625,25 +1667,52 @@ void set_clipboard_text(struct history_info *h, GList *element)
 			action=g_strdup("mousedown 2 && xdotool mouseup 2'");
 		else if(get_pref_int32("auto_key"))
 			action=g_strdup("key ctrl+v'");
-	}
 	
-	if( get_pref_int32("key_input")) 
-		action=g_strconcat("type \"",txt,"\"'",NULL);
+	
+		if( get_pref_int32("key_input")) {
+			gchar *xtxt;
+			int i, e, sl=strlen(txt);
+			for (i=e=0; i<sl; ++i){
+				if( 0x27 == txt[i] || 0x22 == txt[i])
+					++e;
+			}
+				
+			if(NULL == (xtxt=malloc(sl+e+2)))
+				goto done; /**system is likely about to die...  */
+			for (i=e=0; i<sl; ++i){
+				if(0x27 == txt[i] || 0x22 == txt[i])
+					xtxt[e++]='\\';
+			  xtxt[e++]=txt[i];
+			}
+			xtxt[e]=0;
+			/*action=g_strconcat("type \"",xtxt,"\"'",NULL); */
+			action=g_strconcat("txt=",xtxt,NULL);
+			free(xtxt);
+		}
+	}	
 		
 	if(NULL == action)
 			goto done;
-	/**from clipit 1.4.1 */
-  cmd = g_strconcat("/bin/sh -c 'xdotool ", action, NULL);
-	g_fprintf(stderr,"xdotool:'%s'\ntext:'%s'\n",cmd,txt);
+	if(NULL == environ)
+		goto done;
+	env=get_environment(action);
+	if(NULL == env)
+		goto done;
+/*  cmd = g_strconcat("/bin/sh -c 'xdotool ", action, NULL); */
+	cmd = g_strconcat("/bin/sh -c 'xdotool type \"$txt\"'", NULL);
+/*	g_fprintf(stderr,"xdotool:#%s#\ntext:#%s#\n",cmd,txt);  */
   GPid pid;
   gchar **argv;
-  g_shell_parse_argv(cmd, NULL, &argv, NULL);
+	
+	g_shell_parse_argv(cmd, NULL, &argv, NULL);
   g_free(cmd);
-  g_spawn_async(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL);
+  g_spawn_async(NULL, argv, env, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL);
   g_child_watch_add(pid, (GChildWatchFunc)action_exit, NULL);
   g_strfreev(argv);
   /**end from clipit 1.4.1 */
 done:
+	if(NULL != env)
+		g_free(env);
 	if(NULL != txt)
 		g_free(txt);
 	if(NULL != action)
@@ -1682,7 +1751,7 @@ static gboolean my_item_event (GtkWidget *w,GdkEventKey *e, gpointer user)
 	}
 	if(GDK_KEY_PRESS == e->type){
 		/*GdkEventKey *k=	(GdkEventKey *)e; */
-		printf("key press %d (0x%x)\n",e->keyval,e->keyval); 
+		g_fprintf(stderr,"key press %d (0x%x)\n",e->keyval,e->keyval); 
 		fflush(NULL);
 	}
 	if(GDK_BUTTON_RELEASE==e->type){
@@ -1731,7 +1800,7 @@ static void item_selected(GtkMenuItem *menu_item, gpointer user_data)
 	GdkEventKey *k=(GdkEventKey *)gtk_get_current_event();
 	GList* element = g_list_nth(history_list, GPOINTER_TO_INT(user_data));
 	/*g_print ("item_selected '%s' type %x val %x\n",(gchar *)((struct history_item *(element->data))->text),k->type, k->keyval);  */
-	if(0xFF0d == k->keyval && GDK_KEY_PRESS == k->type){
+	if((0xFF0d == k->keyval || 0xFF8d == k->keyval ) && GDK_KEY_PRESS == k->type){
 		set_clipboard_text(h,element);
 	}
 }	
@@ -1794,6 +1863,28 @@ void destroy_history_menu(GtkMenuShell *menu, gpointer u)
 	selection_done(menu,u);	/**allow deleted items to be deleted.  */
 	gtk_widget_destroy((GtkWidget *)menu);
 }
+
+/***************************************************************************/
+/** 
+****************************************************************************/
+void set_menu_xy(GtkMenu *menu, gint *x, gint *y, gboolean *push, gpointer user_data)
+{
+  GdkDisplay *dis;
+  if (NULL != (dis=gdk_display_get_default())) {
+    gdk_display_get_pointer(dis, NULL,x,y,NULL);
+    if( NULL == user_data)
+    	*y=1; // history
+    else
+      *y-=120; //icon menu
+    *x-=50;
+    if(*x<=0)
+    	*x=1;
+		if(*y<=0)
+			*y=1;
+  }
+
+}
+
 /***************************************************************************/
 /**  Called when status icon is left-clicked or action key hit.
 \n\b Arguments:
@@ -1932,7 +2023,7 @@ static gboolean show_history_menu(gpointer data)
       if ((clipboard_temp) && (p_strcmp(hist_text, clipboard_temp) == 0))
       {
         gchar* bold_text = g_markup_printf_escaped("<b>%s</b>", string->str);
-			  if( NULL == bold_text) g_fprintf(stderr,"NulBMKUp:'%s'\n",string->str);
+			  /*if( NULL == bold_text) g_fprintf(stderr,"NulBMKUp:'%s'\n",string->str); */
         gtk_label_set_markup((GtkLabel*)item_label, bold_text);
         g_free(bold_text);
         h.clip_item=menu_item;
@@ -1942,7 +2033,7 @@ static gboolean show_history_menu(gpointer data)
       else if ((primary_temp) && (p_strcmp(hist_text, primary_temp) == 0))
       {
         gchar* italic_text = g_markup_printf_escaped("<i>%s</i>", string->str);
-			  if( NULL == italic_text) g_fprintf(stderr,"NulIMKUp:'%s'\n",string->str);
+			  /*if( NULL == italic_text) g_fprintf(stderr,"NulIMKUp:'%s'\n",string->str); */
         gtk_label_set_markup((GtkLabel*)item_label, italic_text);
         g_free(italic_text);
         h.clip_item=menu_item;
@@ -2005,7 +2096,10 @@ next_loop:
 	
 	if(get_pref_int32("type_search")){
     /* Edit clipboard */
-		h.title_item = gtk_image_menu_item_new_with_label( _("Use Alt-E to edit, Alt-C to clear") );
+		if( get_pref_int32("disable_clear"))
+			h.title_item = gtk_image_menu_item_new_with_label( _("Use Alt-E to edit") );
+		else
+				h.title_item = gtk_image_menu_item_new_with_label( _("Use Alt-E to edit, Alt-C to clear") );
     menu_image = gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
     gtk_image_menu_item_set_image((GtkImageMenuItem*)h.title_item, menu_image);
     gtk_menu_shell_append((GtkMenuShell*)menu, h.title_item);    
@@ -2028,7 +2122,7 @@ next_loop:
 	g_signal_connect(menu,"selection-done",(GCallback)destroy_history_menu,(gpointer)&h);
   /* Popup the menu... */
   gtk_widget_show_all(menu);
-  gtk_menu_popup((GtkMenu*)menu, NULL, NULL, get_pref_int32("history_pos")?postition_history:NULL, NULL, 1, gtk_get_current_event_time());
+  gtk_menu_popup((GtkMenu*)menu, NULL, NULL, get_pref_int32("history_pos")?postition_history:set_menu_xy, NULL, 1, gtk_get_current_event_time());
 	/**set last entry at first -fixes bug 2974614 */
 	if(get_pref_int32("reverse_history") && NULL != h.clip_item)
 		gtk_menu_shell_select_item((GtkMenuShell*)menu,h.clip_item);
@@ -2066,6 +2160,7 @@ void _show_history_menu (GtkMenuItem *m, gpointer data)
 {
 	g_timeout_add(POPUP_DELAY, show_history_menu, GINT_TO_POINTER(figure_histories()));
 }
+
 /***************************************************************************/
 /** .
 \n\b Arguments:
@@ -2075,6 +2170,7 @@ GtkWidget *create_parcellite_menu(guint button, guint activate_time)
 {
   /* Declare some variables */
   GtkWidget *menu, *menu_item;
+  gint x,y;
   
   /* Create the menu */
   menu = gtk_menu_new();
@@ -2110,8 +2206,8 @@ GtkWidget *create_parcellite_menu(guint button, guint activate_time)
   gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
   /* Popup the menu... */
   gtk_widget_show_all(menu);
-  gtk_menu_popup((GtkMenu*)menu, NULL, NULL, NULL, NULL, button, activate_time);	
-	return menu;
+  gtk_menu_popup((GtkMenu*)menu, NULL, NULL, set_menu_xy, (gpointer)1, button, activate_time);
+  return menu;
 }
 
 /* Called when status icon is right-clicked */
@@ -2234,7 +2330,7 @@ static void parcellite_init()
 	if(FALSE ==g_thread_supported()){
 		g_fprintf(stderr,"g_thread not init!\n");
 	}
-	hist_lock= g_mutex_new();
+	g_mutex_init(&hist_lock);
   
   show_icon=!get_pref_int32("no_icon");
   /* Read history */
@@ -2355,7 +2451,7 @@ int main(int argc, char *argv[])
 		mode=PROG_MODE_CLIENT; /**already running, just access fifos & exit.  */
 	
 	/**get options/cmd line not parsed.  */
-	if( NULL != opts->leftovers)g_print("%s\n",opts->leftovers);
+	if( NULL != opts->leftovers)g_fprintf(stderr,"%s\n",opts->leftovers);
 	/**init fifo should set up the fifo and the callback (if we are daemon mode)  */
 		if(opts->primary)	{
 			if(NULL == (fifo=init_fifo(FIFO_MODE_PRI|mode))) return 1;
