@@ -21,7 +21,7 @@
 #include "preferences.h"
 G_BEGIN_DECLS
 
-extern GMutex *hist_lock;
+extern GMutex hist_lock;
 
 #define ACTIONS_TAB    2
 #define POPUP_DELAY    100
